import os
import unittest
from dotenv import dotenv_values

from aws_s3_helper import S3

class TestS3Operations(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        env = dotenv_values()
        cls.s3 = S3(
            aws_access_key_id=env["AWS_ACCESS_KEY_ID"],
            aws_secret_access_key=env["AWS_SECRET_ACCESS_KEY"],
            aws_region=env["AWS_REGION"],
        )
        cls.bucket_name = "bucket-test"
        cls.new_bucket_name = "bucket-test-renamed"
        cls.local_folder = "test_folder"
        cls.file_name = "test_file.txt"
        cls.s3_path = "test_folder/"
        cls.new_s3_path = "test_folder_renamed/"
        cls.text_file = "Hello, this is a test"
        cls.local_file_path = os.path.join(cls.local_folder, cls.file_name)

        # Create local folder and test file
        os.makedirs(cls.local_folder, exist_ok=True)
        with open(cls.local_file_path, "w") as f:
            f.write(cls.text_file)

    def test_0_create_bucket(self):
        # Create a new bucket
        self.s3.create_bucket(self.bucket_name)
        buckets = self.s3.list_buckets()
        self.assertIn(self.bucket_name, buckets)

    def test_1_upload_file(self):
        # Upload a single file to S3
        self.s3.upload_file(self.bucket_name, self.local_file_path, self.s3_path + self.file_name)

        # Check that the uploaded file exists in S3
        self.assertTrue(
            self.file_name in self.s3.list(self.bucket_name, self.s3_path)["files"]
        )

    def test_2_download_file(self):
        # Remove the local file before downloading
        if os.path.exists(self.local_file_path):
            os.remove(self.local_file_path)

        # Download the file from S3
        self.s3.download_file(self.bucket_name, self.s3_path + self.file_name, self.local_file_path)

        # Check that the file has been downloaded and the contents are correct
        self.assertTrue(os.path.exists(self.local_file_path))
        with open(self.local_file_path, "r") as f:
            content = f.read()
        self.assertEqual(content, self.text_file)

    def test_3_upload_folder(self):
        # Upload the folder to S3
        self.s3.upload_folder(self.bucket_name, self.local_folder, self.s3_path)

        # Check that the uploaded folder and its file exist in S3
        self.assertTrue(
            self.file_name in self.s3.list(self.bucket_name, self.s3_path)["files"]
        )

    def test_4_rename_file(self):
        # Rename the file in S3
        new_file_name = "test_file_renamed.txt"
        self.s3.rename_file(self.bucket_name, self.s3_path + self.file_name, self.s3_path + new_file_name)

        # Check that the old file does not exist and the new one does
        self.assertFalse(
            self.file_name in self.s3.list(self.bucket_name, self.s3_path)["files"]
        )
        self.assertTrue(
            new_file_name in self.s3.list(self.bucket_name, self.s3_path)["files"]
        )

        # Update the file name for subsequent tests
        self.file_name = new_file_name

    def test_5_rename_folder(self):
        # Rename the folder in S3
        self.s3.rename_folder(self.bucket_name, self.s3_path, self.new_s3_path)

        # Check that the old folder does not exist and the new one does
        self.assertFalse(
            self.s3_path in self.s3.list(self.bucket_name)["folders"]
        )
        self.assertTrue(
            self.new_s3_path in self.s3.list(self.bucket_name)["folders"]
        )

        # Update the S3 path for subsequent tests
        self.s3_path = self.new_s3_path

    def test_6_download_folder(self):
        # Remove the local folder before downloading
        if os.path.exists(self.local_folder):
            os.system(f"rm -rf {self.local_folder}")

        # Download the folder from S3
        self.s3.download_folder(self.bucket_name, self.s3_path, self.local_folder)

        # Check that the folder has been downloaded and the contents are correct
        self.assertTrue(os.path.exists(os.path.join(self.local_folder, self.file_name)))
        with open(os.path.join(self.local_folder, self.file_name), "r") as f:
            content = f.read()
        self.assertEqual(content, self.text_file)

    def test_7_delete_folder(self):
        # Remove the folder in S3
        self.s3.delete_folder(self.bucket_name, self.s3_path)

        # Check that the folder has been deleted in S3
        self.assertTrue(self.s3_path not in self.s3.list(self.bucket_name)["folders"])

    def test_8_rename_bucket(self):
        # Rename the bucket
        self.s3.rename_bucket(self.bucket_name, self.new_bucket_name)
        
        # Check that the old bucket no longer exists and the new bucket does
        buckets = self.s3.list_buckets()
        self.assertNotIn(self.bucket_name, buckets)
        self.assertIn(self.new_bucket_name, buckets)

        # Update the bucket name for subsequent tests
        self.bucket_name = self.new_bucket_name

    def test_9_delete_bucket(self):
        # Delete the bucket created during the test
        self.s3.delete_bucket(self.bucket_name)
        buckets = self.s3.list_buckets()
        self.assertNotIn(self.bucket_name, buckets)

    @classmethod
    def tearDownClass(cls):
        # Cleans up the local files created
        if os.path.exists(cls.local_folder):
            os.system(f"rm -rf {cls.local_folder}")


if __name__ == "__main__":
    unittest.main()


from aws_s3_helper.interface import S3Interface
from aws_s3_helper.s3_with_logs import S3WithLogs
from aws_s3_helper.s3_without_logs import S3WithoutLogs


class S3(S3Interface):
    def __init__(self, aws_region: str, aws_access_key_id: str = None, aws_secret_access_key: str = None, logging: bool = False):
        """
        AWS S3 client.

        Args:
            aws_access_key_id (str): The AWS access key.
            aws_secret_access_key (str, optional): The AWS secret access key.
            aws_region (str, optional): The AWS region.
            logging (bool, optional): If True, the client will show logs.

        Raises:
            Exception: If the credentials or region are not well defined.
        """
        self.__client = (
            S3WithLogs(
                aws_access_key_id=aws_access_key_id,
                aws_secret_access_key=aws_secret_access_key,
                aws_region=aws_region,
            )
            if logging
            else S3WithoutLogs(
                aws_access_key_id=aws_access_key_id,
                aws_secret_access_key=aws_secret_access_key,
                aws_region=aws_region,
            )
        )

    # ------------------------- List functions ---------------------------

    def list_buckets(self) -> list[str]:
        return self.__client.list_buckets()

    def list(self, bucket_name: str, prefix: str = "") -> dict[str, list[str]]:
        return self.__client.list(bucket_name, prefix)

    # ------------------------- File functions ---------------------------

    def read_file(self, bucket_name: str, s3_path: str, format: str | None = None) -> str:
        return self.__client.read_file(bucket_name, s3_path, format)

    def rename_file(self, bucket_name: str, old_s3_path: str, new_s3_path: str) -> None:
        self.__client.rename_file(bucket_name, old_s3_path, new_s3_path)

    def download_file(self, bucket_name: str, s3_path: str, local_path: str) -> None:
        self.__client.download_file(bucket_name, s3_path, local_path)

    def get_presigned_url_file(self, bucket_name: str, s3_path: str) -> str:
        return self.__client.get_presigned_url_file(bucket_name, s3_path)

    def upload_file(self, bucket_name: str, file_path: str, s3_path: str) -> None:
        self.__client.upload_file(bucket_name, file_path, s3_path)

    def delete_file(self, bucket_name: str, s3_path: str) -> None:
        self.__client.delete_file(bucket_name, s3_path)

    # ------------------------- Folder functions ---------------------------

    def download_folder(self, bucket_name: str, s3_path: str, local_path: str) -> None:
        self.__client.download_folder(bucket_name, s3_path, local_path)

    def upload_folder(self, bucket_name: str, folder_path: str, s3_path: str) -> None:
        self.__client.upload_folder(bucket_name, folder_path, s3_path)

    def delete_folder(self, bucket_name: str, s3_path: str) -> None:
        self.__client.delete_folder(bucket_name, s3_path)

    def rename_folder(self, bucket_name: str, old_s3_path: str, new_s3_path: str) -> None:
        self.__client.rename_folder(bucket_name, old_s3_path, new_s3_path)

    # ------------------------- Buckets functions ---------------------------

    def create_bucket(self, bucket_name: str) -> None:
        self.__client.create_bucket(bucket_name)

    def delete_bucket(self, bucket_name: str) -> None:
        self.__client.delete_bucket(bucket_name)

    def rename_bucket(self, old_bucket_name: str, new_bucket_name: str) -> None:
        self.__client.rename_bucket(old_bucket_name, new_bucket_name)



import os

import boto3

from aws_s3_helper.interface import S3Interface
from aws_s3_helper.s3_base import S3Base

class S3WithoutLogs(S3Interface, S3Base):
    def __init__(
        self, aws_access_key_id: str, aws_secret_access_key: str, aws_region: str
    ):
        """
        AWS S3 client without logging.

        Args:
            aws_access_key_id: The AWS access key.
            aws_secret_access_key: The AWS secret access key.
            aws_region: The AWS region.

        Raises:
            Exception: If the credentials or region are not well defined.
        """
        try:
            self.__client = boto3.client(
                "s3",
                aws_access_key_id=aws_access_key_id,
                aws_secret_access_key=aws_secret_access_key,
                region_name=aws_region,
            )
            self.__region = aws_region
            
            super().__init__(self.__client)
        except Exception as e:
            raise Exception("--- The S3 configuration is not correct --- ERROR", e)

    # ------------------------- List functions ---------------------------

    def list_buckets(self) -> list[str]:
        try:
            response = self.__client.list_buckets()
            buckets = [bucket["Name"] for bucket in response["Buckets"]]
            return buckets
        except Exception as e:
            raise Exception("Failed to list buckets.", e)

    def list(self, bucket_name: str, prefix: str = "") -> dict[str, list[str]]:
        if prefix != "":
            # Removes the leading slash from the prefix if it exists
            if prefix.startswith("/"):
                prefix = prefix[1:]
        
            # Add a trailing slash to the prefix if it does not exist
            if not prefix.endswith("/"):
                prefix = prefix + "/"
            
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")

        if len(prefix) > 0 and not self._s3_path_exists(bucket_name, prefix):
            raise Exception(
                f"The path '{prefix}' does not exist in the bucket '{bucket_name}'."
            )

        paginator = self.__client.get_paginator("list_objects_v2")
        operation_parameters: dict[str, str] = {
            "Bucket": bucket_name,
            "Prefix": prefix,
            "Delimiter": "/",
        }

        files: list[str] = []
        folders: list[str] = []

        for page in paginator.paginate(**operation_parameters):
            folders.extend(
                [
                    f["Prefix"][len(prefix) :].rstrip("/")
                    for f in page.get("CommonPrefixes", [])
                ]
            )
            files.extend([f["Key"][len(prefix) :] for f in page.get("Contents", [])])

        return {"folders": folders, "files": files}

    # ------------------------- File functions ---------------------------

    def read_file(self, bucket_name: str, s3_path: str, format: str | None = None) -> str:
        # Removes the leading slash from the s3_path if it exists
        if s3_path.startswith("/"):
            s3_path = s3_path[1:]
            
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")

        if not self._s3_path_exists(bucket_name, s3_path):
            raise Exception(
                f"The file '{s3_path}' does not exist in the bucket '{bucket_name}'."
            )

        response = self.__client.get_object(Bucket=bucket_name, Key=s3_path)

        if format is None:
            content = response["Body"].read()
        else:
            content = response["Body"].read().decode(format)
        return content

    def rename_file(self, bucket_name: str, old_s3_path: str, new_s3_path: str) -> None:
        # Removes the leading slash from the old_s3_path or new_s3_path if it exists
        if old_s3_path.startswith("/"):
            old_s3_path = old_s3_path[1:]
        if new_s3_path.startswith("/"):
            new_s3_path = new_s3_path[1:]
        
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")

        if not self._s3_path_exists(bucket_name, old_s3_path):
            raise Exception(f"The file '{old_s3_path}' does not exist in the bucket '{bucket_name}'.")

        if self._s3_path_exists(bucket_name, new_s3_path):
            raise Exception(f"A file already exists at the new path '{new_s3_path}' in the bucket '{bucket_name}'.")

        # Copy the file to the new path
        self.__client.copy_object(
            Bucket=bucket_name,
            CopySource={'Bucket': bucket_name, 'Key': old_s3_path},
            Key=new_s3_path
        )

        # Delete the old file
        self.delete_file(bucket_name, old_s3_path)
    
    def download_file(self, bucket_name: str, s3_path: str, local_path: str) -> None:
        # Removes the leading slash from the s3_path if it exists
        if s3_path.startswith("/"):
            s3_path = s3_path[1:]
        
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")

        if not self._s3_path_exists(bucket_name, s3_path):
            raise Exception(
                f"The file '{s3_path}' does not exist in the bucket '{bucket_name}'."
            )
            
        self.__client.download_file(bucket_name, s3_path, local_path)
        
    def get_presigned_url_file(self, bucket_name: str, s3_path: str) -> str:
        # Removes the leading slash from the s3_path if it exists
        if s3_path.startswith("/"):
            s3_path = s3_path[1:]
        
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")
        
        if not self._s3_path_exists(bucket_name, s3_path):
            raise Exception(
                f"The file '{s3_path}' does not exist in the bucket '{bucket_name}'."
            )
        
        response = self.__client.generate_presigned_url(
            ClientMethod="get_object",
            Params={
                "Bucket": bucket_name,
                "Key": s3_path
            }
        )
        
        return response

    def upload_file(self, bucket_name: str, file_path: str, s3_path: str) -> None:
        # Removes the leading slash from the prefix if it exists
        if s3_path.startswith("/"):
            s3_path = s3_path[1:]
        
        if not os.path.isfile(file_path):
            raise FileNotFoundError(
                f"The file '{file_path}' does not exist on the local file system."
            )

        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")

        self.__client.upload_file(
            Filename=file_path,
            Bucket=bucket_name,
            Key=s3_path
        )
        
    def delete_file(self, bucket_name: str, s3_path: str) -> None:
        # Removes the leading slash from the s3_path if it exists
        if s3_path.startswith("/"):
            s3_path = s3_path[1:]
        
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")

        if not self._s3_path_exists(bucket_name, s3_path):
            raise Exception(
                f"The file '{s3_path}' does not exist in the bucket '{bucket_name}'."
            )

        self.__client.delete_object(Bucket=bucket_name, Key=s3_path)

    # ------------------------- Folder functions ---------------------------

    def download_folder(self, bucket_name: str, s3_path: str, local_path: str) -> None:
        # Removes the leading slash from the s3_path if it exists
        if s3_path.startswith("/"):
            s3_path = s3_path[1:]
        
        # Add a trailing slash to the s3_path if it does not exist
        if not s3_path.endswith("/"):
            s3_path = s3_path + "/"
            
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")

        if not self._s3_path_exists(bucket_name, s3_path):
            raise Exception(
                f"The folder '{s3_path}' does not exist in the bucket '{bucket_name}'."
            )

        # Create the local folder if it does not exist
        os.makedirs(local_path, exist_ok=True)
        objects = self.list(bucket_name, s3_path)

        for file in objects["files"]:
            local_file_path = os.path.join(local_path, file)
            os.makedirs(os.path.dirname(local_file_path), exist_ok=True)
            self.download_file(
                bucket_name, os.path.join(s3_path, file), local_file_path
            )

        for folder in objects["folders"]:
            self.download_folder(
                bucket_name,
                os.path.join(s3_path, folder) + "/",
                os.path.join(local_path, folder) + "/",
            )

    def upload_folder(self, bucket_name: str, folder_path: str, s3_path: str) -> None:
        # Removes the leading slash from the s3_path if it exists
        if s3_path.startswith("/"):
            s3_path = s3_path[1:]
        
        # Add a trailing slash to the s3_path if it does not exist
        if not s3_path.endswith("/"):
            s3_path = s3_path + "/"
        
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")

        if not os.path.isdir(folder_path):
            raise FileNotFoundError(
                f"The folder '{folder_path}' does not exist on the local file system."
            )

        for root, dirs, files in os.walk(folder_path):
            for file in files:
                local_file_path = os.path.join(root, file)
                relative_path = os.path.relpath(local_file_path, folder_path)
                s3_file_path = os.path.join(s3_path, relative_path).replace(
                    "\\", "/"
                )

                self.upload_file(bucket_name, local_file_path, s3_file_path)

            for folder in dirs:
                s3_folder_path = (
                    os.path.join(
                        s3_path,
                        os.path.relpath(os.path.join(root, folder), folder_path),
                    ).replace("\\", "/")
                    + "/"
                )
                self.__client.put_object(Bucket=bucket_name, Key=s3_folder_path)

    def delete_folder(self, bucket_name: str, s3_path: str) -> None:
        # Removes the leading slash from the s3_path if it exists
        if s3_path.startswith("/"):
            s3_path = s3_path[1:]
        
        # Add a trailing slash to the s3_path if it does not exist
        if not s3_path.endswith("/"):
            s3_path = s3_path + "/"
        
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")

        if not self._s3_path_exists(bucket_name, s3_path):
            raise Exception(
                f"The folder '{s3_path}' does not exist in the bucket '{bucket_name}'."
            )

        objects = self.list(bucket_name, s3_path)
        
        for file in objects["files"]:
            self.delete_file(bucket_name, os.path.join(s3_path, file))

        for folder in objects["folders"]:
            self.delete_folder(bucket_name, os.path.join(s3_path, folder) + "/")
                
    def rename_folder(self, bucket_name: str, old_s3_path: str, new_s3_path: str) -> None:
        # Removes the leading slash from the old_s3_path or new_s3_path if it exists
        if old_s3_path.startswith("/"):
            old_s3_path = old_s3_path[1:]
        if new_s3_path.startswith("/"):
            new_s3_path = new_s3_path[1:]
        
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")

        if not self._s3_path_exists(bucket_name, old_s3_path):
            raise Exception(f"The folder '{old_s3_path}' does not exist in the bucket '{bucket_name}'.")

        if self._s3_path_exists(bucket_name, new_s3_path):
            raise Exception(f"A folder already exists at the new path '{new_s3_path}' in the bucket '{bucket_name}'.")

        objects = self.list(bucket_name, old_s3_path)

        for file in objects["files"]:
            old_file_path = os.path.join(old_s3_path, file)
            new_file_path = os.path.join(new_s3_path, file)
            self.rename_file(bucket_name, old_file_path, new_file_path)


        for folder in objects["folders"]:
            old_folder_path = os.path.join(old_s3_path, folder) + "/"
            new_folder_path = os.path.join(new_s3_path, folder) + "/"
            self.rename_folder(bucket_name, old_folder_path, new_folder_path)


        # Optionally, delete the original folder's "directory marker"
        self.delete_file(bucket_name, old_s3_path)
         
    # ------------------------- Buckets functions ---------------------------
    
    def create_bucket(self, bucket_name: str) -> None:
        # Remove the leading slash from the bucket_name if it exists
        bucket_name.replace("/", "")
        
        if self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' already exists.")

        try:
            self.__client.create_bucket(
                Bucket=bucket_name,
                CreateBucketConfiguration={"LocationConstraint": self.__region}
            )
        except self.__client.exceptions.BucketAlreadyExists:
            raise Exception(f"The bucket name '{bucket_name}' is already taken globally.")
        except self.__client.exceptions.BucketAlreadyOwnedByYou:
            raise Exception(f"You already own a bucket named '{bucket_name}'.")
        except Exception as e:
            raise Exception(f"Failed to create bucket '{bucket_name}'.", e)
        
    def delete_bucket(self, bucket_name: str) -> None:
        # Remove the leading slash from the bucket_name if it exists
        bucket_name.replace("/", "")
        
        if not self._bucket_exists(bucket_name):
            raise Exception(f"The bucket '{bucket_name}' does not exist.")
        
        objects = self.list(bucket_name)
        if objects["files"] or objects["folders"]:
            raise Exception(f"The bucket '{bucket_name}' is not empty.")
        
        self.__client.delete_bucket(Bucket=bucket_name)
    
    def rename_bucket(self, old_bucket_name: str, new_bucket_name: str) -> None:
        # Remove the leading slash from the bucket_name if it exists
        old_bucket_name.replace("/", "")
        new_bucket_name.replace("/", "")
        
        if not self._bucket_exists(old_bucket_name):
            raise Exception(f"The bucket '{old_bucket_name}' does not exist.")

        if self._bucket_exists(new_bucket_name):
            raise Exception(f"The bucket '{new_bucket_name}' already exists.")

        # Create the new bucket
        self.create_bucket(new_bucket_name)

        # Copy all objects from the old bucket to the new bucket
        objects = self.list(old_bucket_name)
        total_items = len(objects["files"]) + len(objects["folders"])

        # Copy files
        for file in objects["files"]:
            copy_source = {'Bucket': old_bucket_name, 'Key': file}
            self.__client.copy_object(
                Bucket=new_bucket_name,
                CopySource=copy_source,
                Key=file
            )

        # Copy folders (as empty keys)
        for folder in objects["folders"]:
            folder_path = folder + "/" if not folder.endswith("/") else folder
            self.__client.put_object(Bucket=new_bucket_name, Key=folder_path)

        # Delete the old bucket and its contents
        self.delete_bucket(old_bucket_name)


from abc import ABC, abstractmethod

class S3Interface(ABC):
    """
    Interface for the S3 class.
    """

    @abstractmethod
    def list_buckets(self) -> list[str]:
        """
        Lists all the buckets in the AWS S3 account.

        Returns:
            list[str]: A list of bucket names.
            
        Raises:
            Exception: If there is an error listing the buckets.
        """
        ...

    @abstractmethod
    def list(self, bucket_name: str, prefix: str = "") -> dict[str, list[str]]:
        """
        Lists folders and files in an AWS S3 bucket under a specific prefix.

        Args:
            bucket_name (str): The name of the S3 bucket.
            prefix (str, optional): The prefix used to filter the objects in the bucket.
                                    Default is an empty string. Must be find with '/'

        Returns:
            dict: A dictionary with two lists:
                  - 'folders': Names of folders found under the prefix (without the prefix or trailing slash).
                  - 'files': Names of the files found under the prefix (without the prefix).
                  
        Raises:
            Exception: If the bucket does not exist or if there is an error listing the objects.
        """
        pass
    
    # ------------------------- File functions ---------------------------
    
    @abstractmethod
    def read_file(self, bucket_name: str, s3_path: str, format: str | None = None) -> str:
        """
        Reads the contents of a file stored in an AWS S3 bucket without downloading it.

        Args:
            bucket_name (str): The name of the S3 bucket.
            s3_path (str): The path to the file in the bucket.
            format (str, optional): The format of the file. Default is None.

        Returns:
            str: The contents of the file as a string without encoding or decoding if format is provided.

        Raises:
            Exception: If the bucket or file does not exist.
        """
        pass
    
    @abstractmethod
    def rename_file(self, bucket_name: str, old_s3_path: str, new_s3_path: str) -> None:
        """
        Renames a file in an AWS S3 bucket.

        Args:
            bucket_name (str): The name of the S3 bucket.
            old_s3_path (str): The current path of the file in the bucket.
            new_s3_path (str): The new path of the file in the bucket.

        Raises:
            Exception: If the bucket or file does not exist, or if the new path already exists.
        """
        pass
    
    @abstractmethod
    def download_file(self, bucket_name: str, s3_path: str, local_path: str) -> None:
        """
        Download a file from an AWS S3 bucket.

        Args:
            bucket_name (str): The name of the S3 bucket.
            s3_path (str): The path to the file in the bucket.
            local_path (str): The path where the downloaded file will be saved.
            
        Raises:
            Exception: If the bucket or file does not exist.
        """
        pass
    
    @abstractmethod
    def get_presigned_url_file(self, bucket_name: str, s3_path: str) -> str:
        """
        Get a presigned URL for a file in an AWS S3 bucket. This URL can be used to download the file without authentication.

        Args:
            bucket_name (str): The name of the S3 bucket.
            s3_path (str): The path to the file in the bucket.
            
        Raises:
            Exception: If the bucket or file does not exist.
        """
        pass
    
    @abstractmethod
    def upload_file(self, bucket_name: str, file_path: str, s3_path: str) -> None:
        """
        Upload a file to an AWS S3 bucket with a progress bar, checking if the file exists locally.
        
        Args:
            bucket_name (str): The name of the S3 bucket.
            file_path (str): The local path of the file to upload.
            s3_path (str): The path in the bucket where the file will be saved.

        Raises:
            Exception: If the bucket does not exist or the local file is not found.
        """
        pass
    
    @abstractmethod
    def delete_file(self, bucket_name: str, s3_path: str) -> None:
        """
        Deletes a file from an AWS S3 bucket.

        Args:
            bucket_name (str): The name of the S3 bucket.
            s3_path (str): The path to the file in the bucket.

        Raises:
            Exception: If the bucket or file does not exist.
        """
        pass
    
     # ------------------------- Folder functions ---------------------------
    
    @abstractmethod
    def download_folder(self, bucket_name: str, s3_path: str, local_path: str) -> None:
        """
        Download a folder from an AWS S3 bucket.

        Args:
            bucket_name (str): The name of the S3 bucket.
            s3_path (str): The path to the folder in the bucket. Must be find with '/'
            local_path (str): The path where the downloaded folder will be saved.
            
        Raises:
            Exception: If the bucket or folder does not exist.
        """
        pass

    @abstractmethod
    def upload_folder(self, bucket_name, s3_path, local_path):
        """
        Uploads a folder to an S3 bucket.

        Args:
            bucket_name (str): The name of the S3 bucket.
            s3_path (str): The path to the folder in the bucket. Must be find with '/'
            local_path (str): The path where the uploaded folder is saved.

        Raises:
            Exception: If the bucket or folder does not exist.
        """
        pass
    
    @abstractmethod
    def delete_folder(self, bucket_name, s3_path):
        """
        Deletes a folder and all its contents from an S3 bucket.

        Args:
            bucket_name (str): The name of the S3 bucket.
            s3_path (str): The path to the folder in the bucket. Must be find with '/'

        Raises:
            Exception: If the bucket or folder does not exist.
        """
        pass

    @abstractmethod
    def rename_folder(self, bucket_name, old_s3_path, new_s3_path):
        """
        Renames a folder in an S3 bucket.

        Args:
            bucket_name (str): The name of the S3 bucket.
            old_s3_path (str): The current path of the folder in the bucket. Must be find with '/'
            new_s3_path (str): The new path of the folder in the bucket. Must be find with '/'

        Raises:
            Exception: If the bucket or folder does not exist, or if the new path already exists.
        """
        pass
    
    # ------------------------- Buckets functions ---------------------------
    
    @abstractmethod
    def create_bucket(self, bucket_name):
        """
        Creates an S3 bucket.

        Args:
            bucket_name (str): The name of the S3 bucket.

        Raises:
            Exception: If the bucket already exists or if there is an error during creation.
        """
        pass

    @abstractmethod
    def delete_bucket(self, bucket_name):
        """
        Deletes an S3 bucket.

        Args:
            bucket_name (str): The name of the S3 bucket.

        Raises:
            Exception: If the bucket does not exist or is not empty.
        """
        pass

    @abstractmethod
    def rename_bucket(self, old_bucket_name, new_bucket_name):
        """
        Renames an S3 bucket.

        Args:
            old_bucket_name (str): The name of the existing S3 bucket.
            new_bucket_name (str): The new name for the S3 bucket.

        Raises:
            Exception: If the old bucket does not exist, if the new bucket already exists,
                    or if there is an error during the process.
        """
        pass


from botocore.exceptions import ClientError

class S3Base:
    """
    Common methods for the S3 class.
    """
    def __init__(self, client) -> None:
        self.__client = client
    
    
    def _bucket_exists(self, bucket_name: str) -> bool:
        """
        Checks if an S3 bucket exists.

        Args:
            bucket_name (str): The name of the S3 bucket.

        Returns:
            bool: True if the bucket exists, False if it does not.

        Raises:
            Exception: If an error occurs when trying to verify the existence of the bucket.
        """
        try:
            self.__client.head_bucket(Bucket=bucket_name)
            return True
        except ClientError as e:
            error_code = int(e.response["Error"]["Code"])
            if error_code == 404:
                return False
            else:
                raise Exception(f"Error verifying the existence of the bucket: {e}")


    def _s3_path_exists(self, bucket_name: str, s3_path: str) -> bool:
        """
        Checks if a file or folder exists in the S3 bucket under the specified path.

        Args:
            bucket_name (str): The name of the S3 bucket.
            s3_path (str): The path to the file or folder in the bucket.

        Returns:
            bool: True if the file or folder exists, False if it does not exist.

        Raises:
            Exception: If an error occurs when trying to verify the existence of the file or folder.
        """
        if  s3_path.endswith("/"):  # Folder
            result = self.__client.list_objects_v2(
                Bucket=bucket_name, Prefix=s3_path, Delimiter="/"
            )
            return "CommonPrefixes" in result or "Contents" in result
        else:  # File
            try:
                self.__client.head_object(Bucket=bucket_name, Key=s3_path)
                return True
            except ClientError as e:
                error_code = int(e.response["Error"]["Code"])
                if error_code == 404:
                    return False
                else:
                    raise Exception(
                        f"Error verifying the existence of the file or folder: {e}"
                    )


from setuptools import find_packages, setup

REQUIREMENTS_FILE = "requirements.txt"
SRC_FOLDER = "src"

with open("README.md", "r", encoding="utf-8") as fh:
    long_description: str = fh.read()

setup(
    name="aws-s3-helper",
    version="1.3.2",
    author="Sergio Jimenez Moreno",
    author_email="sergio.jimenez@xauencybersecurity.com",
    description="Abstraction for easy access to AWS S3.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/sjm00010/AWS_S3_Helper",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    package_dir={"": SRC_FOLDER},
    packages=find_packages(where=SRC_FOLDER),
    install_requires=["boto3", "tqdm"],
    python_requires=">=3.8",
)
